<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<!-- Generated with PHPDoc, http://www.phpdoc.de -->
<html>
<head>
	<title>{APPNAME} - Report</title>
	<link href="phpdoc.css" rel="stylesheet" type="text/css">
</head>

<body>
<table width="100%" border="0">
	<tr>
		<td align="left" valign="top" bgcolor="#B0BAFF">			
			<table width="100%">
				<tr>
					<td align="left" valign="top"><a href="phpdoc_packagelist.html">Packageindex</a></td>
					<td align="left" valign="top"><a href="phpdoc_classtree.html">Classtrees</a></td>
					<td align="left" valign="top"><a href="phpdoc_modulegroup.html">Modulegroups</a></td>
					<td align="left" valign="top"><a href="phpdoc_elementlist.html">Elementlist</a></td>
					<td align="left" valign="top"><a href="phpdoc_warnings.html">Report</a></td>
					<td align="left" valign="top"><a href="phpdoc_xmlfiles.html">XML Files</a></td>
				</tr>
			</table>	
		</td>
	</tr>
	<tr>	
		<td height="20" align="left" valign="top"><hr></td>
	</tr>
	<tr>
		<td align="left" valign="top"><h1>Warnings</h1></td>
	</tr
	<!-- BEGIN warning -->
	<tr>
		<td height="10" align="left" valign="top">&nbsp;</td>
	</tr>
	<tr>
		<td align="left" valign="top">
			<h4>{FILE}</h4>
			{NUMWARNINGS} Warning(s).
		</td>
	</tr>
	<tr>
		<td height="10" align="left" valign="top">&nbsp;</td>
	</tr>
	<tr>
		<td align="left" valign="top">
			<table border="0" cellspacing="4" cellpadding="4" bordercolor="Silver">
				<!-- BEGIN warning_loop -->
				<tr>
					<td align="left" valign="top"><b>[{ELEMENTTYPE}] {ELEMENT}</b></td>
					<td align="left" valign="top">{WARNINGTYPE}</td>
				</tr>
				<tr>
					<td colspan="2" align="left" valign="top">{WARNING}</td>
				</tr>
				<tr>
					<td height="5" colspan="2" align="left" valign="top">&nbsp;</td>
				</tr>
				<!-- END warning_loop -->
			</table>
		</td>
	</tr>
	<!-- END warning -->
	<tr>
		<td height="10" align="left" valign="top"><hr></td>
	</tr>
	<tr>
		<td align="left" valign="top" bgcolor="#C0C0C0">			
			<table width="100%">
				<tr>
					<td align="left" valign="top"><a href="phpdoc_packagelist.html">Packageindex</a></td>
					<td align="left" valign="top"><a href="phpdoc_classtree.html">Classtrees</a></td>
					<td align="left" valign="top"><a href="phpdoc_modulegroup.html">Modulegroups</a></td>
					<td align="left" valign="top"><a href="phpdoc_elementlist.html">Elementlist</a></td>
					<td align="left" valign="top"><a href="phpdoc_warnings.html">Report</a></td>
					<td align="left" valign="top"><a href="phpdoc_xmlfiles.html">XML Files</a></td>
				</tr>
			</table>	
		</td>
	</tr>
	<tr>
		<td align="left" valign="top">PHPDoc 1.0beta</td>
	</tr>
</table>
</body>
</html>
