<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<!-- Generated with PHPDoc, http://www.phpdoc.de -->
<html>
<head>
	<title>{APPNAME} - Frameset Packagelist</title>
	<link href="phpdoc.css" rel="stylesheet" type="text/css">
</head>

<body>

<table width="100%">
	<tr>
		<td align="left" valign="top"><h4>Packagelist</h4></td>
	</tr>
	<!-- BEGIN package -->
	<tr>
		<td align="left" valign="top">{PACKAGE}</td>
	</tr>
	<!-- END package -->
</table>

</body>
</html>
