<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<!-- Generated with PHPDoc, http://www.phpdoc.de -->
<html>
<head>
	<title>{APPNAME} - {CLASS_PACKAGE} - Class: {CLASS_NAME}</title>
	<link href="phpdoc.css" rel="stylesheet" type="text/css">
</head>

<body>
<table width="100%" border="0">
	<tr>
		<td align="left" valign="top" bgcolor="#B0BAFF">			
			<table width="100%">
				<tr>
					<td align="left" valign="top"><a href="phpdoc_packagelist.html">Packageindex</a></td>
					<td align="left" valign="top"><a href="phpdoc_classtree.html">Classtrees</a></td>
					<td align="left" valign="top"><a href="phpdoc_modulegroup.html">Modulegroups</a></td>
					<td align="left" valign="top"><a href="phpdoc_elementlist.html">Elementlist</a></td>
					<td align="left" valign="top"><a href="phpdoc_warnings.html">Report</a></td>
					<td align="left" valign="top"><a href="phpdoc_xmlfiles.html">XML Files</a></td>
				</tr>
			</table>	
		</td>
	</tr>
	<tr>	
		<td height="20" align="left" valign="top"><hr></td>
	</tr>
	<tr>
		<td align="left" valign="top">			
			File: <a href="file://{CLASS_FILE}" target="_new">{CLASS_FILE}</a>
			<h5>{APPNAME} - {CLASS_PACKAGE}</h5>
			<h1>{CLASS_NAME}</h1>
			{CLASS_TREE}
			<p>
			{CLASS_SHORTDESCRIPTION}{CLASS_UNDOC}
			<p>
			&nbsp;
		</td>
	</tr>
	<tr>
		<td align="left" valign="top">
			<h4>{CLASS_ACCESS} {CLASS_ABSTRACT} {CLASS_STATIC} {CLASS_FINAL} class {CLASS_NAME} {CLASS_EXTENDS}</h4>
			{CLASS_DESCRIPTION}
			<p>
			<table>
				<!-- BEGIN class_see --><tr><td align="left" valign="top">See Also</td><td align="left" valign="top">{SEE}</td></tr><!-- END class_see -->
				<!-- BEGIN class_links --><tr><td align="left" valign="top">Links</td><td align="left" valign="top">{LINK}</td></tr><!-- END class_links -->
				<!-- BEGIN class_authors --><tr><td align="left" valign="top">Authors</td><td align="left" valign="top">{AUTHOR}</td></tr><!-- END class_authors -->
				<!-- BEGIN class_since --><tr><td align="left" valign="top">Since</td><td align="left" valign="top">{SINCE}</td></tr><!-- END class_since -->
				<!-- BEGIN class_version --><tr><td align="left" valign="top">Version</td><td align="left" valign="top">{VERSION}</td></tr><!-- END class_version -->
				<!-- BEGIN class_deprecated --><tr><td align="left" valign="top">Deprecated</td><td align="left" valign="top">{DEPRECATED}</td></tr><!-- END class_deprecated -->
				<!-- BEGIN class_copyright --><tr><td align="left" valign="top">Copyright</td><td align="left" valign="top">{COPYRIGHT}</td></tr><!-- END class_copyright -->
				<!-- BEGIN class_magic --><tr><td align="left" valign="top">Magic</td><td align="left" valign="top">{MAGIC}</td></tr><!-- END class_magic -->
			</table>
		</td>
	</tr>
	<tr>
		<td height="10" align="left" valign="top">&nbsp;</td>
	</tr>
	<!-- BEGIN subclasses -->
	<tr>
		<td align="left" valign="top">
			<b>Direct known subclasses:</b> {ELEMENTS}
		</td>
	</tr>
	<!-- END subclasses -->
	<tr>
		<td height="20" align="left" valign="top"><hr></td>
	</tr>
	<!-- BEGIN inheritedfunctions -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td align="left" valign="top"><h3>Methods inherited from {SOURCE}</h3></td>
				</tr>
				<tr>
					<td align="left" valign="top">{ELEMENTS}</td>
				</tr>
			</table>
		</td>
	</tr>
	<!-- END inheritedfunctions -->
	<!-- BEGIN functionsummary -->
	<tr>
		<td align="left" valign="top">		
			<table width="100%" border="0">
				<tr>
					<td colspan="2" align="left" valign="top"><h2>{ACCESS} Method Summary</h2></td>
				</tr>
				<!-- BEGIN functionsummary_loop -->
				<tr>
					<td align="right" valign="top">{RETURNTYPE}</td>
					<td align="left" valign="top" width="100%"><a href="#function_{NAME}"><h4>{NAME}({PARAMETER})</h4></a>{SHORTDESCRIPTION}{UNDOC}</td>
				</tr>
				<!-- END functionsummary_loop -->
			</table>
		</td>
	</tr>
	<!-- END functionsummary -->
	<!-- BEGIN inheritedvariables -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td align="left" valign="top"><h3>Fields inherited from {SOURCE}</h3></td>
				</tr>
				<tr>
					<td align="left" valign="top">{ELEMENTS}</td>
				</tr>
			</table>
		</td>
	</tr>
	<!-- END inheritedvariables -->
	<!-- BEGIN variablesummary -->
	<tr>
		<td align="left" valign="top">		
			<table width="100%" border="0">
				<tr>
					<td colspan="2" align="left" valign="top"><h2>{ACCESS} Field Summary</h2></td>
				</tr>
				<!-- BEGIN variablesummary_loop -->
				<tr>
					<td align="right" valign="top">{TYPE}</td>
					<td align="left" valign="top" width="100%"><a href="#variable_{NAME}"><h4>{NAME}</h4></a>{SHORTDESCRIPTION}{UNDOC}</td>
				</tr>
				<!-- END variablesummary_loop -->
			</table>
		</td>
	</tr>
	<!-- END variablesummary -->
	<!-- BEGIN inheritedconstants -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td align="left" valign="top"><h3>Constants inherited from {SOURCE}</h3></td>
				</tr>
				<tr>
					<td align="left" valign="top">{ELEMENTS}</td>
				</tr>
			</table>
		</td>
	</tr>
	<!-- END inheritedconstants -->
	<!-- BEGIN constantssummary -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td align="left" valign="top" colspan=3><h3>{ACCESS} Constant Summary</h3></td>
				</tr>
				<!-- BEGIN constantssummary_loop -->
				<tr>
					<td align="left" valign="top"><a href="#constants_{NAME}">{NAME}</a></td>
					<td align="left" valign="top">>><code>{VALUE}</code><<</td>
					<td align="left" valign="top" width="100%">{SHORTDESCRIPTION}{UNDOC}</td>
				</tr>
				<!-- END constantssummary_loop -->
			</table>
		</td>
	</tr>
	<!-- END constantssummary -->
	<!-- BEGIN inheriteduses -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td align="left" valign="top"><h3>Included files inherited from {SOURCE}</h3></td>
				</tr>
				<tr>
					<td align="left" valign="top">{ELEMENTS}</td>
				</tr>
			</table>
		</td>
	</tr>
	<!-- END inheriteduses -->
	<!-- BEGIN usessummary -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td align="left" valign="top" colspan=2><h3>Included files Summary, Type: {TYPE}</h3></td>
				</tr>
				<!-- BEGIN usessummary_loop -->
				<tr>
					<td align="left" valign="top"><a href="#uses_{FILE}">{FILE}</a></td>
					<td align="left" valign="top" width="100%"><a href="#uses_{FILE}">{SHORTDESCRIPTION}{UNDOC}</a></td>
				</tr>
				<!-- END usessummary_loop -->
			</table>
		</td>
	</tr>
	<!-- END usessummary -->
	<!-- BEGIN functiondetails -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td colspan="2" align="left" valign="top"><h2>{ACCESS} Method Details</h2></td>
				</tr>
				<!-- BEGIN functiondetails_loop -->
				<tr><td colspan="2" align="left" valign="top"><a name="function_{NAME}"><h4>{NAME}</h4></a></td></tr>
				<tr>
					<td colspan="2" align="left" valign="top">
						<b>{ACCESS} {ABSTRACT} {STATIC} {FINAL} {RETURNTYPE} {NAME}( {PARAMETER} )</b><p>
					</td>
				</tr>
				<tr>
					<td width="10" align="left" valign="top">&nbsp;</td>
					<td align="left" valign="top">
						{DESCRIPTION}
						<p>
						{UNDOC}
					</td>
				</tr>
				<!-- BEGIN functiondetails_parameter -->
				<tr>
					<td colspan="2" align="left" valign="top">Parameter</td>
				</tr>
				<!-- BEGIN functiondetails_parameter_loop -->
				<tr>
					<td></td>
					<td align="left" valign="top">						
						<table>
							<tr>
								<td align="left" valign="top">{TYPE}</td>
								<td align="left" valign="top">{NAME}</td>
								<td align="left" valign="top"><code>{DEFAULT}</code></td>
							</tr>
							<tr>
								<td></td>
								<td colspan="2" align="left" valign="top">{DESCRIPTION}{UNDOC}</td>
							</tr>
						</table>
					</td>
				</tr>
				<!-- END functiondetails_parameter_loop -->
				<!-- END functiondetails_parameter -->
				<!-- BEGIN functiondetails_return -->
				<tr>
					<td align="left" valign="top">Returns</td>
					<td>{TYPE} {NAME}<p>{DESCRIPTION}</td>
				</tr>
				<!-- END functiondetails_return -->
				<!-- BEGIN functiondetails_throws -->
				<tr>
					<td align="left" valign="top">Throws</td>
					<td align="left" valign="top">{EXCEPTIONS}</td>
				</tr>
				<!-- END functiondetails_throws -->
				<!-- BEGIN functiondetails_globals -->
				<tr>
					<td colspan="2" align="left" valign="top">Required global variables</td>
				</tr>
				<!-- BEGIN functiondetails_globals_loop -->
				<tr>
					<td align="left" valign="top"></td>
					<td align="left" valign="top">
						<table>
							<tr>
								<td align="left" valign="top">{TYPE} {NAME}</td>
								<td align="left" valign="top">{DESCRIPTION}</td>
							</tr>
						</table>
					</td>
				</tr>
				<!-- END functiondetails_globals_loop -->
				<!-- END functiondetails_globals -->
				<!-- BEGIN functiondetails_version -->
				<tr>
					<td align="left" valign="top">Version</td>
					<td align="left" valign="top">{VERSION}</td>
				</tr>
				<!-- END functiondetails_version -->
				<!-- BEGIN functiondetails_since -->
				<tr>
					<td align="left" valign="top">Since</td>
					<td align="left" valign="top">{SINCE}</td>
				</tr>
				<!-- END functiondetails_since -->
				<!-- BEGIN functiondetails_deprecated -->
				<tr>
					<td align="left" valign="top">Deprecated</td>
					<td align="left" valign="top">{DEPRECATED}</td>
				</tr>
				<!-- END functiondetails_deprecated -->
				<!-- BEGIN functiondetails_copyright -->
				<tr>
					<td align="left" valign="top">Copyright</td>
					<td align="left" valign="top">{COPYRIGHT}</td>
				</tr>
				<!-- END functiondetails_copyright -->
				<!-- BEGIN functiondetails_magic -->
				<tr>
					<td align="left" valign="top">Magic</td>
					<td align="left" valign="top">{MAGIC}</td>
				</tr>
				<!-- END functiondetails_magic -->
				<!-- BEGIN functiondetails_brother -->
				<tr>
					<td align="left" valign="top">Brother</td>
					<td align="left" valign="top">{BROTHER}</td>
				</tr>
				<!-- END functiondetails_brother -->
				<!-- BEGIN functiondetails_see -->
				<tr>
					<td align="left" valign="top">See Also</td>
					<td align="left" valign="top">{SEE}</td>
				</tr>
				<!-- END functiondetails_see -->
				<!-- BEGIN functiondetails_links -->
				<tr>
					<td align="left" valign="top">Links</td>
					<td align="left" valign="top">{LINK}</td>
				</tr>
				<!-- END functiondetails_links -->
				<!-- BEGIN functiondetails_authors -->
				<tr>
					<td align="left" valign="top">Author(s)</td>
					<td align="left" valign="top">{AUTHOR}</td>
				</tr>
				<!-- END functiondetails_authors -->
				<tr>
					<td colspan="2" align="left" valign="top"><hr></td>
				</tr>
				<!-- END functiondetails_loop -->
			</table>
	</tr>
	<!-- END functiondetails -->	
	<!-- BEGIN variabledetails -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td colspan="2" align="left" valign="top"><h2>{ACCESS} Field Details</h2></td>
				</tr>
				<!-- BEGIN variabledetails_loop -->
				<tr><td colspan="2" align="left" valign="top"><a name="variable_{NAME}"><h4>{NAME}</h4></a></td></tr>
				<tr>
					<td colspan="2" align="left" valign="top">
						<b>{ACCESS} {STATIC} {FINAL} {TYPE} {NAME}</b><p>
						>><code>{VALUE}</code><<
					</td>
				</tr>
				<!-- BEGIN variabledetails_version -->
				<tr>
					<td align="left" valign="top">Version</td>
					<td align="left" valign="top">{VERSION}</td>
				</tr>
				<!-- END variabledetails_version -->
				<!-- BEGIN variabledetails_since -->
				<tr>
					<td align="left" valign="top">Since</td>
					<td align="left" valign="top">{SINCE}</td>
				</tr>
				<!-- END variabledetails_since -->
				<!-- BEGIN variabledetails_deprecated -->
				<tr>
					<td align="left" valign="top">Deprecated</td>
					<td align="left" valign="top">{DEPRECATED}</td>
				</tr>
				<!-- END variabledetails_deprecated -->
				<!-- BEGIN variabledetails_copyright -->
				<tr>
					<td align="left" valign="top">Copyright</td>
					<td align="left" valign="top">{COPYRIGHT}</td>
				</tr>
				<!-- END variabledetails_copyright -->
				<!-- BEGIN variabledetails_magic -->
				<tr>
					<td align="left" valign="top">Magic</td>
					<td align="left" valign="top">{MAGIC}</td>
				</tr>
				<!-- END variabledetails_magic -->
				<!-- BEGIN variabledetails_brother -->
				<tr>
					<td align="left" valign="top">Brother</td>
					<td align="left" valign="top">{BROTHER}</td>
				</tr>
				<!-- END variabledetails_brother -->
				<!-- BEGIN variabledetails_see -->
				<tr>
					<td align="left" valign="top">See Also</td>
					<td align="left" valign="top">{SEE}</td>
				</tr>
				<!-- END variabledetails_see -->
				<!-- BEGIN variabledetails_links -->
				<tr>
					<td align="left" valign="top">Links</td>
					<td align="left" valign="top">{LINK}</td>
				</tr>
				<!-- END variabledetails_links -->
				<!-- BEGIN variabledetails_authors -->
				<tr>
					<td align="left" valign="top">Author(s)</td>
					<td align="left" valign="top">{AUTHOR}</td>
				</tr>
				<!-- END variabledetails_authors -->
				<tr>
					<td colspan="2" align="left" valign="top"><hr></td>
				</tr>
				<!-- END variabledetails_loop -->
			</table>
	</tr>
	<!-- END variabledetails -->	
	<!-- BEGIN constantdetails -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td colspan="2" align="left" valign="top"><h2>{ACCESS} Constant Details</h2></td>
				</tr>
				<!-- BEGIN constantdetails_loop -->
				<tr>
					<td colspan="2" align="left" valign="top">
						<a name="constant_{NAME}"><h4>{NAME}</h4></a>
						<code>define( {NAME}, >>{VALUE}<< )</code><br>
						Case: {CASE}
					</td>
				</tr>
				<!-- BEGIN constantdetails_version -->
				<tr>
					<td align="left" valign="top">Version</td>
					<td align="left" valign="top">{VERSION}</td>
				</tr>
				<!-- END constantdetails_version -->
				<!-- BEGIN constantdetails_since -->
				<tr>
					<td align="left" valign="top">Since</td>
					<td align="left" valign="top">{SINCE}</td>
				</tr>
				<!-- END constantdetails_since -->
				<!-- BEGIN constantdetails_deprecated -->
				<tr>
					<td align="left" valign="top">Deprecated</td>
					<td align="left" valign="top">{DEPRECATED}</td>
				</tr>
				<!-- END constantdetails_deprecated -->
				<!-- BEGIN constantdetails_copyright -->
				<tr>
					<td align="left" valign="top">Copyright</td>
					<td align="left" valign="top">{COPYRIGHT}</td>
				</tr>
				<!-- END constantdetails_copyright -->
				<!-- BEGIN constantdetails_magic -->
				<tr>
					<td align="left" valign="top">Magic</td>
					<td align="left" valign="top">{MAGIC}</td>
				</tr>
				<!-- END constantdetails_magic -->
				<!-- BEGIN constantdetails_brother -->
				<tr>
					<td align="left" valign="top">Brother</td>
					<td align="left" valign="top">{BROTHER}</td>
				</tr>
				<!-- END constantdetails_brother -->
				<!-- BEGIN constantdetails_see -->
				<tr>
					<td align="left" valign="top">See Also</td>
					<td align="left" valign="top">{SEE}</td>
				</tr>
				<!-- END constantdetails_see -->
				<!-- BEGIN constantdetails_links -->
				<tr>
					<td align="left" valign="top">Links</td>
					<td align="left" valign="top">{LINK}</td>
				</tr>
				<!-- END constantdetails_links -->
				<!-- BEGIN constantdetails_authors -->
				<tr>
					<td align="left" valign="top">Author(s)</td>
					<td align="left" valign="top">{AUTHOR}</td>
				</tr>
				<!-- END constantdetails_authors -->
				<tr>
					<td colspan="2" align="left" valign="top"><hr></td>
				</tr>
				<!-- END constantdetails_loop -->
			</table>
	</tr>
	<!-- END constantdetails -->
	<!-- BEGIN usesdetails -->
	<tr>
		<td align="left" valign="top">
			<table width="100%" border="0">
				<tr>
					<td colspan="2" align="left" valign="top"><h2>Included Files, Type: {TYPE}</h2></td>
				</tr>
				<!-- BEGIN usesdetails_loop -->
				<tr><td colspan="2" align="left" valign="top"><a name="uses_{FILE}"><h4>{FILE}</h4></a></td></tr>
				<tr>
					<td colspan="2" align="left" valign="top">
						<b>{TYPE}( {FILE} )</b>
					</td>
				</tr>
				<!-- BEGIN usesdetails_version -->
				<tr>
					<td align="left" valign="top">Version</td>
					<td align="left" valign="top">{VERSION}</td>
				</tr>
				<!-- END usesdetails_version -->
				<!-- BEGIN usesdetails_since -->
				<tr>
					<td align="left" valign="top">Since</td>
					<td align="left" valign="top">{SINCE}</td>
				</tr>
				<!-- END usesdetails_since -->
				<!-- BEGIN usesdetails_deprecated -->
				<tr>
					<td align="left" valign="top">Deprecated</td>
					<td align="left" valign="top">{DEPRECATED}</td>
				</tr>
				<!-- END usesdetails_deprecated -->
				<!-- BEGIN usesdetails_copyright -->
				<tr>
					<td align="left" valign="top">Copyright</td>
					<td align="left" valign="top">{COPYRIGHT}</td>
				</tr>
				<!-- END usesdetails_copyright -->
				<!-- BEGIN usesdetails_magic -->
				<tr>
					<td align="left" valign="top">Magic</td>
					<td align="left" valign="top">{MAGIC}</td>
				</tr>
				<!-- END usesdetails_magic -->
				<!-- BEGIN usesdetails_brother -->
				<tr>
					<td align="left" valign="top">Brother</td>
					<td align="left" valign="top">{BROTHER}</td>
				</tr>
				<!-- END usesdetails_brother -->
				<!-- BEGIN usesdetails_see -->
				<tr>
					<td align="left" valign="top">See Also</td>
					<td align="left" valign="top">{SEE}</td>
				</tr>
				<!-- END usesdetails_see -->
				<!-- BEGIN usesdetails_links -->
				<tr>
					<td align="left" valign="top">Links</td>
					<td align="left" valign="top">{LINK}</td>
				</tr>
				<!-- END usesdetails_links -->
				<!-- BEGIN usesdetails_authors -->
				<tr>
					<td align="left" valign="top">Author(s)</td>
					<td align="left" valign="top">{AUTHOR}</td>
				</tr>
				<!-- END usesdetails_authors -->
				<tr>
					<td colspan="2" align="left" valign="top"><hr></td>
				</tr>
				<!-- END usesdetails_loop -->
			</table>
	</tr>
	<!-- END usesdetails -->
	<tr>
		<td height="10" align="left" valign="top"><hr></td>
	</tr>
	<tr>
		<td align="left" valign="top" bgcolor="#C0C0C0">			
			<table>
				<tr>
					<td align="left" valign="top"><a href="phpdoc_packagelist.html">Packageindex</a></td>
					<td align="left" valign="top"><a href="phpdoc_classtree.html">Classtrees</a></td>
					<td align="left" valign="top"><a href="phpdoc_modulegroup.html">Modulegroups</a></td>
					<td align="left" valign="top"><a href="phpdoc_elementlist.html">Elementlist</a></td>
					<td align="left" valign="top"><a href="phpdoc_warnings.html">Report</a></td>
					<td align="left" valign="top"><a href="phpdoc_xmlfiles.html">XML Files</a></td>
				</tr>
			</table>	
		</td>
	</tr>
	<tr>
		<td align="left" valign="top">PHPDoc 1.0beta</td>
	</tr>
</table>

</body>
</html>
