<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<!-- Generated with PHPDoc, http://www.phpdoc.de -->
<html>
<head>
	<link href="phpdoc.css" rel="stylesheet" type="text/css">
	<title>{APPNAME} - Frameset Packageelements</title>
</head>

<body>
	<table width="100%">
		<tr>
			<td colspan=3 align="left" valign="top">
				<h3>Packagelist</h3>
			</td>
		</tr>
		<!-- BEGIN package -->
		<tr>
			<td colspan="3" align="left" valign="top"><h4>{PACKAGE_NAME}</h4></td>
		</tr>
		<!-- BEGIN package_class -->
		<tr>
			<td align="left" valign="top">&nbsp;</td>
			<td colspan="2" align="left" valign="top"><h5>Classes</h5>{EMPTY}</td>
		</tr>
		<!-- BEGIN package_class_loop -->
		<tr>
			<td colspan="2" align="left" valign="top">&nbsp;</td>
			<td align="left" valign="top">{ELEMENT}</td>
		</tr>
		<!-- END package_class_loop -->
		<!-- END package_class -->
		<!-- BEGIN package_module -->
		<tr>
			<td align="left" valign="top">&nbsp;</td>
			<td colspan="2" align="left" valign="top"><h5>Modules</h5>{EMPTY}</td>
		</tr>
		<!-- BEGIN package_module_loop -->
		<tr>
			<td colspan="2" align="left" valign="top">&nbsp;</td>
			<td align="left" valign="top">{ELEMENT}</td>
		</tr>
		<!-- END package_module_loop -->
		<!-- END package_module -->
		<!-- END package -->
	</table>
</body>
</html>
