/*
   +----------------------------------------------------------------------+
   | PHP version 4.0                                                      |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: Sam Ruby (rubys@us.ibm.com)                                  |
   +----------------------------------------------------------------------+
*/

/* $Id: formatter.java,v 1.1.1.2 2000/09/07 00:06:36 wsanchez Exp $ */

package net.php;

import javax.servlet.*;
import javax.servlet.http.*;

public class formatter extends servlet {

    public void service(HttpServletRequest request,
                        HttpServletResponse response) 
       throws ServletException
    {
       display_source_mode = true;
       super.service(request, response);
    }

}
